import logging

from immutable_views import DictView
from workspaces.shared_interfaces import MessageArchitectIF
from workspaces.workflow.schema_interfaces import WorkflowRequestIF

logger = logging.getLogger(__name__)

"""
Class which create an immutable dictionary of amqp message templates for easy access
"""

workflow_msg_templates = DictView(
    {
        "workflow_failed": {
            "service": "workflow",
            "routing_key": "workflow",
            "subject": None,
            "type": "workflow-failed",
        },
        "qa_ready": {
            "service": "capability",
            "routing_key": "capability",
            "subject": None,
            "type": "qa_ready",
        },
        "delivery": {
            "subject": None,
            "service": "workflow",
            "type": "delivery",
            "delivery": None,
        },
        "ingestion_complete": {
            "service": "workflow",
            "routing_key": "workflow",
            "subject": None,
            "type": "ingestion-complete",
        },
    }
)

archive_msg_templates = DictView(
    {
        "carta_ready": {
            "service": "archive",
            "routing_key": None,
            "application": "workflow",
            "carta_url": None,
            "subject": None,
            "message": "carta ready",
        },
        "carta_failed": {
            "service": "archive",
            "routing_key": None,
            "application": "workflow",
            "carta_url": None,
            "subject": None,
            "message": "carta failed",
        },
        "seci_failed": {
            "service": "archive",
            "routing_key": None,
            "application": "workflow",
            "subject": None,
            "message": "seci failed",
            "status": "failed",
        },
        "seci_complete": {
            "service": "archive",
            "routing_key": None,
            "application": "workflow",
            "subject": None,
            "message": "seci complete",
            "status": "complete",
        },
    }
)


class WorkflowMessageArchitect(MessageArchitectIF):
    def __init__(
        self,
        request: WorkflowRequestIF = None,
        previous_info=None,
        delivery_info=None,
    ):
        self.subject = request or previous_info
        self.delivery = delivery_info

    @staticmethod
    def get_message_template(msg_type: str) -> DictView:
        return workflow_msg_templates[msg_type]

    def compose_message(self, msg_type: str) -> DictView:
        template = self.get_message_template(msg_type)
        template = template.copy()
        template["subject"] = self.subject

        if self.delivery is not None:
            template["delivery"] = self.delivery
        return DictView(template)


class ArchiveMessageArchitect(MessageArchitectIF):
    def __init__(self, routing_key: str, request: WorkflowRequestIF, carta_url: str = None):
        self.key = routing_key
        self.subject = request
        self.carta_url = carta_url

    @staticmethod
    def get_message_template(msg_type: str) -> DictView:
        return archive_msg_templates[msg_type]

    def compose_message(self, msg_type: str) -> DictView:
        template = self.get_message_template(msg_type)
        template = template.copy()
        template["subject"] = self.subject
        template["routing_key"] = self.key

        if self.carta_url is not None:
            template["carta_url"] = self.carta_url

        return DictView(template)
